@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,8 +33,8 @@ class t_call_record {
 public:
 
 enum t_rel_cause {
-	CS_A_PARTY,
-	CS_B_PARTY,
+	CS_LOCAL_USER,
+	CS_REMOTE_USER,
 	CS_FAILURE
 };
 
@@ -86,9 +86,15 @@ public:
 	// Record successful call answer
 	void answer_call(const t_response *resp);
 	
-	// Record end of a successful call
+	// Record end of a successful call with an explicit cause
 	void end_call(t_rel_cause cause);
 	
+	// Record end of a successful call
+	// If far_end is true, then the far-end ended the call, otherwise
+	// the near-end ended the call. This indication together with the
+	// direction determines the correct cause of the call end.
+	void end_call(bool far_end);
+	
 	string get_rel_cause(void) const;
 	string get_direction(void) const;
 	bool set_rel_cause(const string &cause);
@@ -115,6 +121,9 @@ private:
 	
 	list<t_call_record>	call_records;
 	
+	// Number of missed calls since this counter was cleared
+	int		num_missed_calls;
+	
 public:
 	
 	t_call_history();
@@ -136,7 +145,12 @@ public:
 	bool write_history(string &error_msg) const;
 	
 	void get_history(list<t_call_record> &history);
+	
+	// Clear call history file
 	void clear(bool write = true);
+	
+	int get_num_missed_calls(void) const;
+	void clear_num_missed_calls(void);
 };
 
 extern t_call_history *call_history;
