@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@ public:
 
 	void add_credentials(const t_credentials &c);
 	string encode(void) const;
+	string encode_value(void) const;
+	string encode_env(void) const;
 
 	// Return true if the header contains credentials for a realm/dest
 	bool contains(const string &realm, const t_url &uri) const;
