@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,7 +38,22 @@ string t_hdr_call_id::encode(void) const {
 		s = "Call-ID: ";
 	}
 
-	s += call_id + CRLF;
+	s += encode_value();
+	s += CRLF;
 
 	return s;
 }
+
+string t_hdr_call_id::encode_value(void) const {
+	if (!populated) return "";
+	return call_id;
+}
+
+string t_hdr_call_id::encode_env(void) const {
+	string s;
+	
+	s = "SIP_CALL_ID=";
+	s += encode_value();
+	
+	return s;
+}
