@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,6 +33,16 @@ string t_hdr_content_language::encode(void) const {
 	if (!populated) return s;
 
 	s += "Content-Language: ";
+	s += encode_value();
+	s += CRLF;
+
+	return s;
+}
+
+string t_hdr_content_language::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
 
 	for (list<t_language>::const_iterator i = language_list.begin();
 	     i != language_list.end(); i++)
@@ -41,7 +51,14 @@ string t_hdr_content_language::encode(void) const {
 		s += i->encode();
 	}
 
-	s += CRLF;
+	return s;
+}
 
+string t_hdr_content_language::encode_env(void) const {
+	string s;
+	
+	s = "SIP_CONTENT_LANGUAGE=";
+	s += encode_value();
+	
 	return s;
 }
