@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,6 +41,28 @@ string t_hdr_content_length::encode(void) const {
 		s = "Content-Length: ";
 	}
 
-	s += int2str(length) + CRLF;
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_content_length::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
+
+	s = int2str(length);
+	return s;
+}
+
+string t_hdr_content_length::encode_env(void) const {
+	string s;
+
+	if (!populated) return s;
+
+	s = "SIP_CONTENT_LENGTH=";
+	s += encode_value();
+	
 	return s;
 }
