@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -47,6 +47,17 @@ string t_hdr_event::encode(void) const {
 		s = "Event: ";
 	}
 
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_event::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
+
 	s += event_type;
 
 	if (id.size() > 0) {
@@ -56,6 +67,14 @@ string t_hdr_event::encode(void) const {
 
 	s += param_list2str(event_params);
 
-	s += CRLF;
+	return s;
+}
+
+string t_hdr_event::encode_env(void) const {
+	string s;
+	
+	s = "SIP_EVENT=";
+	s += encode_value();
+	
 	return s;
 }
