@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,13 @@ using namespace std;
 class t_hdr_from : public t_header {
 public:
 	string			display; // display name
+	
+	// The display_override may be set by the UA to display another
+	// name to the user, then the display name received in the
+	// signalling, e.g. a lookup from an address book. This value
+	// does NOT appear in the SIP message.
+	string			display_override;
+	
 	t_url			uri;
 	string			tag;
 	list<t_parameter>	params;
@@ -43,6 +50,11 @@ public:
 	void set_params(const list<t_parameter> &l);
 	void add_param(const t_parameter &p);
 	string encode(void) const;
+	string encode_value(void) const;
+	string encode_env(void) const;
+	
+	// Get the display name to show to the user.
+	string get_display_presentation(void) const;
 };
 
 #endif
