@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,17 @@ string t_hdr_in_reply_to::encode(void) const {
 	if (!populated) return s;
 
 	s = "In-Reply-To: ";
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_in_reply_to::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
+
 	for (list<string>::const_iterator i = call_ids.begin();
 	     i != call_ids.end(); i++)
 	{
@@ -39,6 +50,14 @@ string t_hdr_in_reply_to::encode(void) const {
 		s += *i;
 	}
 
-	s += CRLF;
+	return s;
+}
+
+string t_hdr_in_reply_to::encode_env(void) const {
+	string s;
+	
+	s = "SIP_IN_REPLY_TO=";
+	s += encode_value();
+	
 	return s;
 }
