@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,25 @@ string t_hdr_mime_version::encode(void) const {
 
 	if (!populated) return s;
 
-	s = "MIME-Version: " + version + CRLF;
+	s = "MIME-Version: ";
+	s += encode_value();
+	s += CRLF;
+	
 	return s;
 }
+
+string t_hdr_mime_version::encode_value(void) const {
+	if (!populated) return "";
+
+	return version;
+}
+
+string t_hdr_mime_version::encode_env(void) const {
+	string s;
+
+	s = "SIP_MIME_VERSION=";
+	s += encode_value();
+	
+	return s;
+}
+	
\ No newline at end of file
