@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,6 +34,25 @@ string t_hdr_min_expires::encode(void) const {
 
 	if (!populated) return s;
 
-	s = "Min-Expires: " + ulong2str(time) + CRLF;
+	s = "Min-Expires: ";
+	s += encode_value();
+	s += CRLF;
+	
 	return s;
 }
+
+string t_hdr_min_expires::encode_value(void) const {
+	if (!populated) return "";
+
+	return ulong2str(time);
+}
+
+string t_hdr_min_expires::encode_env(void) const {
+	string s;
+
+	s = "SIP_MIN_EXPIRES=";
+	s += encode_value();
+	
+	return s;
+}
+	
\ No newline at end of file
