@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,16 @@ string t_hdr_proxy_require::encode(void) const {
 	if (!populated) return s;
 
 	s = "Proxy-Require: ";
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_proxy_require::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
 
 	for (list<string>::const_iterator i = features.begin();
 	     i != features.end(); i++)
@@ -40,6 +50,14 @@ string t_hdr_proxy_require::encode(void) const {
 		s += *i;
 	}
 
-	s += CRLF;
+	return s;
+}
+
+string t_hdr_proxy_require::encode_env(void) const {
+	string s;
+	
+	s = "SIP_PROXY_REQUIRE=";
+	s += encode_value();
+	
 	return s;
 }
