@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -47,6 +47,10 @@ public:
 	t_hdr_record_route();
 	void add_route(const t_route &r);
 	string encode(void) const;
+	string encode_list(void) const;
+	string encode_multi_header(void) const;
+	string encode_value(void) const;
+	string encode_env(void) const;
 };
 
 #endif
