@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include "definitions.h"
 #include "hdr_route.h"
+#include "parse_ctrl.h"
 
 t_hdr_route::t_hdr_route() : t_header() {
 	route_to_first_route = false;
@@ -29,11 +30,41 @@ void t_hdr_route::add_route(const t_route &r) {
 }
 
 string t_hdr_route::encode(void) const {
+	return (t_parser::multi_values_as_list ? encode_list() : encode_multi_header());
+}
+
+string t_hdr_route::encode_list(void) const {
 	string s;
 
 	if (!populated) return s;
 
 	s = "Route: ";
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_route::encode_multi_header(void) const {
+	string s;
+
+	if (!populated) return s;
+
+	for (list<t_route>::const_iterator i = route_list.begin();
+	     i != route_list.end(); i++)
+	{
+		s += "Route: ";
+		s += i->encode();
+		s += CRLF;
+	}
+
+	return s;
+}
+
+string t_hdr_route::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
 
 	for (list<t_route>::const_iterator i = route_list.begin();
 	     i != route_list.end(); i++)
@@ -42,6 +73,14 @@ string t_hdr_route::encode(void) const {
 		s += i->encode();
 	}
 
-	s += CRLF;
+	return s;
+}
+
+string t_hdr_route::encode_env(void) const {
+	string s;
+	
+	s = "SIP_ROUTE=";
+	s += encode_value();
+	
 	return s;
 }
