@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,11 +34,27 @@ string t_hdr_rseq::encode(void) const {
 
 	if (!populated) return s;
 
-	s = "RSeq: " + ulong2str(resp_nr);
+	s = "RSeq: ";
+	s += encode_value();
 	s += CRLF;
 	return s;
 }
 
+string t_hdr_rseq::encode_value(void) const {
+	if (!populated) return "";
+
+	return ulong2str(resp_nr);
+}
+
+string t_hdr_rseq::encode_env(void) const {
+	string s;
+	
+	s = "SIP_RSEQ=";
+	s += encode_value();
+	
+	return s;
+}
+
 bool t_hdr_rseq::operator==(const t_hdr_rseq &h) const {
 	return (resp_nr == h.resp_nr);
 }
