@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,8 @@ public:
 	void set_resp_nr(unsigned long l);
 
 	string encode(void) const;
+	string encode_value(void) const;
+	string encode_env(void) const;
 
 	bool operator==(const t_hdr_rseq &h) const;
 };
