@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,6 +37,8 @@ public:
 	t_server(const string &_product, const string &_version,
 		 const string &_comment = "");
 	string encode(void) const;
+	string encode_value(void) const;
+	string encode_env(void) const;
 };
 
 class t_hdr_server : public t_header {
@@ -50,6 +52,8 @@ public:
 	string get_server_info(void) const;
 	
 	string encode(void) const;
+	string encode_value(void) const;
+	string encode_env(void) const;
 };
 
 #endif
