@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,24 @@ string t_hdr_subject::encode(void) const {
 		s = "Subject: ";
 	}
 
-	s += subject + CRLF;
+	s += encode_value();
+	s += CRLF;
+	
 	return s;
 }
+
+string t_hdr_subject::encode_value(void) const {
+	if (!populated) return "";
+
+	return subject;
+}
+
+string t_hdr_subject::encode_env(void) const {
+	string s;
+
+	s = "SIP_SUBJECT=";
+	s += encode_value();
+	
+	return s;
+}
+	
\ No newline at end of file
