@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,10 +41,29 @@ string t_hdr_timestamp::encode(void) const {
 	if (!populated) return s;
 
 	s = "Timestamp: ";
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_timestamp::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
+
 	s += float2str(timestamp, "%.3f");
 
 	if (delay != 0) s += float2str(delay, " %.3f");
 
-	s += CRLF;
+	return s;
+}
+
+string t_hdr_timestamp::encode_env(void) const {
+	string s;
+	
+	s = "SIP_TIMESTAMP=";
+	s += encode_value();
+	
 	return s;
 }
