@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +31,23 @@ string t_hdr_www_authenticate::encode(void) const {
 	if (!populated) return s;
 
 	s += "WWW-Authenticate: ";
-	s += challenge.encode();
+	s += encode_value();
 	s += CRLF;
 
 	return s;
 }
+
+string t_hdr_www_authenticate::encode_value(void) const {
+	if (!populated) return "";
+
+	return challenge.encode();
+}
+
+string t_hdr_www_authenticate::encode_env(void) const {
+	string s;
+	
+	s = "SIP_WWW_AUTHENTICATE=";
+	s += encode_value();
+	
+	return s;
+}
