@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,6 +51,19 @@ string t_parameter::encode(void) const {
 	return s;
 }
 
+bool t_parameter::operator==(const t_parameter &rhs) {
+	return (type == rhs.type && name == rhs.name);
+}
+
+t_parameter str2param(const string &s) {
+	list<string> l = split_on_first(s, '=');
+	if (l.size() == 1) {
+		return t_parameter(s);
+	} else {
+		return t_parameter(trim(l.front()), trim(l.back()));
+	}
+}
+
 string param_list2str(const list<t_parameter> &l) {
 	string s;
 
@@ -63,3 +76,15 @@ string param_list2str(const list<t_parameter> &l) {
 
 	return s;
 }
+
+list<t_parameter> str2param_list(const string &s) {
+	list<t_parameter> result;
+	
+	list<string> l = split(s, ';');
+	for (list<string>::const_iterator i = l.begin(); i != l.end(); i++) {
+		t_parameter p = str2param(trim(*i));
+		result.push_back(p);
+	}
+	
+	return result;
+}
