@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,9 +44,17 @@ enum t_param_type{
 	t_parameter(const string &n, const string &v);
 
 	string encode(void) const;
+	
+	bool operator==(const t_parameter &rhs);
 };
 
+// Decode a parameter
+t_parameter str2param(const string &s);
+
 // Encode a parameter list
 string param_list2str(const list<t_parameter> &l);
 
+// Decode a parameter list
+list<t_parameter> str2param_list(const string &s);
+
 #endif
