@@ -40,6 +40,10 @@
 #define SDP_RTPMAP_SPEEX_WB	"speex/16000"
 #define SDP_RTPMAP_SPEEX_UWB	"speex/32000"
 #define SDP_RTPMAP_ILBC		"iLBC/8000"
+#define SDP_RTPMAP_G726_16	"G726-16/8000"
+#define SDP_RTPMAP_G726_24	"G726-24/8000"
+#define SDP_RTPMAP_G726_32	"G726-32/8000"
+#define SDP_RTPMAP_G726_40	"G726-40/8000"
 #define SDP_RTPMAP_TELEPHONE_EV	"telephone-event/8000"
 
 // Audio codec names
@@ -48,6 +52,10 @@
 #define SDP_AC_NAME_GSM			"GSM"
 #define SDP_AC_NAME_SPEEX		"speex"
 #define SDP_AC_NAME_ILBC		"iLBC"
+#define SDP_AC_NAME_G726_16		"G726-16"
+#define SDP_AC_NAME_G726_24		"G726-24"
+#define SDP_AC_NAME_G726_32		"G726-32"
+#define SDP_AC_NAME_G726_40		"G726-40"
 #define SDP_AC_NAME_TELEPHONE_EV	"telephone-event"
 
 // Check on fmtp parameter values
@@ -228,12 +236,15 @@ public:
 
 	// Get ptime. Returns 0 if ptime is not present
 	unsigned short get_ptime(t_sdp_media_type media_type) const;
-
+	
+	bool get_zrtp_support(t_sdp_media_type media_type) const;
+	
 	void set_ptime(t_sdp_media_type media_type, unsigned short ptime);
 	void set_direction(t_sdp_media_type media_type, t_sdp_media_direction direction);
 	void set_fmtp(t_sdp_media_type media_type, unsigned short codec, const string &fmtp);
 	void set_fmtp_int_param(t_sdp_media_type media_type, unsigned short codec,
 			const string &param, int value);
+	void set_zrtp_support(t_sdp_media_type media_type);
 
 	// Returns a pointer to the first media stream in the list of media
 	// streams having a non-zero port value for the give media type.
