@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -145,7 +145,7 @@ t_tmr_phone::t_tmr_phone(long dur, t_phone_timer ptmr, t_phone *p) : t_timer(dur
 
 void t_tmr_phone::expired(void) {
 	// Call timeout method on the phone for a timer expiry
-	phone->timeout(phone_timer);
+	phone->timeout(phone_timer, get_id());
 }
 
 t_timer *t_tmr_phone::copy(void) const {
@@ -169,6 +169,7 @@ t_phone *t_tmr_phone::get_phone(void) const {
 string t_tmr_phone::get_name(void) const {
 	switch(phone_timer) {
 	case PTMR_REGISTRATION:	return "PTMR_REGISTRATION";
+	case PTMR_NAT_KEEPALIVE: return "PTMR_NAT_KEEPALIVE";
 	}
 
 	return "UNKNOWN";
@@ -217,6 +218,7 @@ string t_tmr_line::get_name(void) const {
 	case LTMR_RE_INVITE_GUARD:	return "LTMR_RE_INVITE_GUARD";
 	case LTMR_100REL_TIMEOUT:	return "LTMR_100REL_TIMEOUT";
 	case LTMR_100REL_GUARD:		return "LTMR_100REL_GUARD";
+	case LTMR_CANCEL_GUARD:		return "LTMR_CANCEL_GUARD";
 	}
 
 	return "UNKNOWN";
@@ -736,7 +738,9 @@ void *timekeeper_sigwait(void *arg) {
 	sigaddset(&sigset, SIGALRM);
 
 	while (true) {
-		sigwait(&sigset, &sig);
+		// When SIGCONT is received after SIGSTOP, sigwait returns
+		// with EINTR ??
+		if (sigwait(&sigset, &sig) == EINTR) continue;
 		evq_timekeeper->interrupt();
 	}
 }
